#ifndef CHOICECONDITION_H_
#define CHOICECONDITION_H_

#include "condition.h"
#include "filter.h"

template<typename T>
class ChoiceCondition: public Condition
{
public:
	typedef T Traits;
	typedef typename Traits::Type Type;

private:
	Type value;

public:
	ChoiceCondition(Type);

	virtual Condition *clone() const;
	virtual bool can_merge(const Condition &, const CompoundCondition &) const;
	virtual void add_lines(std::list<FilterStatement> &) const;
	virtual void add_merged_lines(const std::list<Condition *> &, const CompoundCondition &, std::list<FilterStatement> &) const;
};


struct ClassTraits
{
	typedef std::string Type;
	static const char *get_keyword() { return "Class"; }
};

typedef ChoiceCondition<ClassTraits> ClassCondition;


struct BaseTypeTraits
{
	typedef std::string Type;
	static const char *get_keyword() { return "BaseType"; }
};

typedef ChoiceCondition<BaseTypeTraits> BaseTypeCondition;


template<typename Traits>
ChoiceCondition<Traits>::ChoiceCondition(Type v):
	value(v)
{ }

template<typename Traits>
Condition *ChoiceCondition<Traits>::clone() const
{
	return new ChoiceCondition<Traits>(value);
}

template<typename Traits>
bool ChoiceCondition<Traits>::can_merge(const Condition &other, const CompoundCondition &parent) const
{
	return dynamic_cast<const ChoiceCondition<Traits> *>(&other) && dynamic_cast<const OrCondition *>(&parent);
}

template<typename Traits>
void ChoiceCondition<Traits>::add_lines(std::list<FilterStatement> &st) const
{
	const char *keyword = Traits::get_keyword();
	FilterStatement::add_line(st, Msp::format("%s \"%s\"", keyword, value));
}

template<typename Traits>
void ChoiceCondition<Traits>::add_merged_lines(const std::list<Condition *> &conditions, const CompoundCondition &parent, std::list<FilterStatement> &st) const
{
	if(dynamic_cast<const OrCondition *>(&parent))
	{
		std::string line = Traits::get_keyword();
		for(std::list<Condition *>::const_iterator i=conditions.begin(); i!=conditions.end(); ++i)
			line += Msp::format(" \"%s\"", static_cast<const ChoiceCondition<Traits> *>(*i)->value);
		FilterStatement::add_line(st, line);
	}
}

#endif
