#include "condition.h"
#include "filter.h"

using namespace std;
using namespace Msp;

CompoundCondition::~CompoundCondition()
{
	for(list<Condition *>::const_iterator i=conditions.begin(); i!=conditions.end(); ++i)
		delete *i;
}

void CompoundCondition::clone_to(CompoundCondition &other) const
{
	for(list<Condition *>::const_iterator i=conditions.begin(); i!=conditions.end(); ++i)
		other.add((*i)->clone());
}

void CompoundCondition::add(Condition *cond)
{
	conditions.push_back(cond);
}


Condition *AndCondition::clone() const
{
	AndCondition *result = new AndCondition;
	clone_to(*result);
	return result;
}

void AndCondition::add_lines(list<FilterStatement> &st) const
{
	for(list<Condition *>::const_iterator i=conditions.begin(); i!=conditions.end(); ++i)
		(*i)->add_lines(st);
}


Condition *OrCondition::clone() const
{
	OrCondition *result = new OrCondition;
	clone_to(*result);
	return result;
}

void OrCondition::add_lines(list<FilterStatement> &st) const
{
	bool merge = conditions.size()>1;
	for(list<Condition *>::const_iterator i=conditions.begin(); (merge && ++i!=conditions.end()); )
		merge = conditions.front()->can_merge(**i, *this);

	if(merge)
		conditions.front()->add_merged_lines(conditions, *this, st);
	else
	{
		list<FilterStatement> result;
		for(list<Condition *>::const_iterator i=conditions.begin(); i!=conditions.end(); ++i)
		{
			list<FilterStatement> sub_result = st;
			(*i)->add_lines(sub_result);
			result.splice(result.end(), sub_result);
		}
		swap(result, st);
	}
}


LinkedColorsCondition::LinkedColorsCondition(const Colors &c):
	colors(c)
{ }

Condition *LinkedColorsCondition::clone() const
{
	return new LinkedColorsCondition(colors);
}

void LinkedColorsCondition::add_lines(list<FilterStatement> &st) const
{
	FilterStatement::add_line(st, format("SocketGroup %s", colors.colors));
}


void operator>>(const LexicalConverter &conv, LinkedColorsCondition::Colors &colors)
{
	const string &str = conv.get();
	bool rgb = true;
	for(string::const_iterator i=str.begin(); (rgb && i!=str.end()); ++i)
		rgb = (*i=='R' || *i=='G' || *i=='B');
	if(str.size()>6 || !rgb)
		throw lexical_error(format("conversion of '%s' to LinkedColorsCondition::Colors", str));

	fill(colors.colors, colors.colors+7, '\0');
	copy(str.begin(), str.end(), colors.colors);
}
