#ifndef CONDITION_H_
#define CONDITION_H_

#include <list>
#include <string>
#include <msp/strings/format.h>
#include "rarity.h"

class CompoundCondition;
class FilterStatement;

class Condition
{
protected:
	Condition() { }
public:
	virtual ~Condition() { }

	virtual Condition *clone() const = 0;
	virtual bool can_merge(const Condition &, const CompoundCondition &) const { return false; }
	virtual void add_lines(std::list<FilterStatement> &) const = 0;
	virtual void add_merged_lines(const std::list<Condition *> &, const CompoundCondition &, std::list<FilterStatement> &) const { }
};


class CompoundCondition: public Condition
{
protected:
	std::list<Condition *> conditions;

public:
	virtual ~CompoundCondition();

protected:
	void clone_to(CompoundCondition &) const;

public:
	void add(Condition *);
};


class AndCondition: public CompoundCondition
{
public:
	virtual Condition *clone() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};


class OrCondition: public CompoundCondition
{
public:
	virtual Condition *clone() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};


class LinkedColorsCondition: public Condition
{
public:
	struct Colors
	{
		char colors[7];
	};

private:
	Colors colors;
	
public:
	LinkedColorsCondition(const Colors &);

	virtual Condition *clone() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};

void operator>>(const Msp::LexicalConverter &, LinkedColorsCondition::Colors &);

#endif
