#ifndef RANGECONDITION_H_
#define RANGECONDITION_H_

#include "condition.h"
#include "filter.h"

template<typename T>
class RangeCondition: public Condition
{
public:
	typedef T Traits;
	typedef typename Traits::Type Type;

protected:
	Type min;
	Type max;

public:
	RangeCondition(Type);
	RangeCondition(Type, Type);

	virtual Condition *clone() const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};


struct RarityTraits
{
	typedef Rarity Type;
	static Rarity get_min() { return NORMAL; }
	static Rarity get_max() { return UNIQUE; }
	static const char *get_keyword() { return "Rarity"; }
};

typedef RangeCondition<RarityTraits> RarityCondition;


struct QualityTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 0; }
	static unsigned get_max() { return 20; }
	static const char *get_keyword() { return "Quality"; }
};

typedef RangeCondition<QualityTraits> QualityCondition;


struct ItemLevelTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 0; }
	static unsigned get_max() { return 100; }
	static const char *get_keyword() { return "ItemLevel"; }
};

typedef RangeCondition<ItemLevelTraits> ItemLevelCondition;


struct DropLevelTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 0; }
	static unsigned get_max() { return 100; }
	static const char *get_keyword() { return "DropLevel"; }
};

typedef RangeCondition<DropLevelTraits> DropLevelCondition;


struct WidthTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 1; }
	static unsigned get_max() { return 2; }
	static const char *get_keyword() { return "Width"; }
};

typedef RangeCondition<WidthTraits> WidthCondition;


struct HeightTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 1; }
	static unsigned get_max() { return 4; }
	static const char *get_keyword() { return "Height"; }
};

typedef RangeCondition<HeightTraits> HeightCondition;


struct SocketsTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 1; }
	static unsigned get_max() { return 6; }
	static const char *get_keyword() { return "Sockets"; }
};

typedef RangeCondition<SocketsTraits> SocketsCondition;


struct LinkedSocketsTraits
{
	typedef unsigned Type;
	static unsigned get_min() { return 1; }
	static unsigned get_max() { return 6; }
	static const char *get_keyword() { return "LinkedSockets"; }
};

typedef RangeCondition<LinkedSocketsTraits> LinkedSocketsCondition;


template<typename Traits>
RangeCondition<Traits>::RangeCondition(Type v):
	min(v),
	max(v)
{ }

template<typename Traits>
RangeCondition<Traits>::RangeCondition(Type n, Type x):
	min(n),
	max(x)
{ }

template<typename Traits>
Condition *RangeCondition<Traits>::clone() const
{
	return new RangeCondition<Traits>(min, max);
}

template<typename Traits>
void RangeCondition<Traits>::add_lines(std::list<FilterStatement> &st) const
{
	const char *keyword = Traits::get_keyword();
	if(min==max)
		FilterStatement::add_line(st, Msp::format("%s %s", keyword, min));
	else
	{
		if(min!=Traits::get_min())
			FilterStatement::add_line(st, Msp::format("%s >= %s", keyword, min));
		if(max!=Traits::get_max())
			FilterStatement::add_line(st, Msp::format("%s <= %s", keyword, max));
	}
}

#endif
