#ifndef THEME_H_
#define THEME_H_

#include <map>
#include <string>
#include <msp/datafile/objectloader.h>
#include "appearance.h"
#include "color.h"

class Theme
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Theme>
	{
	public:
		Loader(Theme &);

	private:
		void appearance(const std::string &);
		void color(const std::string &, unsigned, unsigned, unsigned);
		void color_alpha(const std::string &, unsigned, unsigned, unsigned, unsigned);
	};

private:
	typedef std::map<std::string, Color> ColorMap;
	typedef std::map<std::string, Appearance> AppearanceMap;

	ColorMap colors;
	unsigned base_font_size;
	AppearanceMap appearances;

public:
	Theme();

	void set_color(const std::string &, const Color &);
	const Color &get_color(const std::string &) const;

	void set_base_font_size(unsigned);
	unsigned get_base_font_size() const { return base_font_size; }

	void set_appearance(const std::string &, const Appearance &);
	const Appearance &get_appearance(const std::string &) const;
	const Appearance *find_appearance(const std::string &) const;
};

#endif
